<?php
	
	declare(strict_types=1);
	
	/**
	 * @author: daniel
	 * @date: 13.06.24
	 * @time: 15:18
	 */
	
	namespace wpsg;
	
	class GiroPayPalAPI {
		
		private static ?GiroPayPalAPI $instance = null;
		
		private string $client_id;
		private string $client_secret;
		private bool $bSandbox;
		
		public function __construct(string $client_id, string $client_secret, bool $bSandbox) {
		
			$this->client_id = $client_id;
			$this->client_secret = $client_secret;
			$this->bSandbox = $bSandbox;
		
		}
		
		public function captureOrder(string $paypal_order_id): array {
			
			$r = $this->call(
				'/v2/checkout/orders/'.$paypal_order_id.'/capture',
				null
			);
			
			return $r;
			
		}
		
		public function creeateOrder(\wpsg_order $oOrder): string {
			
			$r = $this->call(
				'/v2/checkout/orders',
				[
					'intent' => 'CAPTURE',
					'purchase_units' => [
						[
							'reference_id' => $oOrder->getId(),
							'amount' => [
								'currency_code' => 'EUR',
								'value' => $oOrder->getToPay()
							]
						]
					]
				]
			);
			
			if (!isset($r['id'])) die(wpsg_debug($r));
			
			return $r['id'];
			
		}
		
		private function call(string $path, ?array $arBodyData = null): ?array {
		
			if ($this->bSandbox) $url = 'https://api-m.sandbox.paypal.com'.$path;
			else $url = 'https://api-m.paypal.com'.$path;
			
			$ch = curl_init();
			
			curl_setopt($ch, CURLOPT_URL, $url);
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
			curl_setopt($ch, CURLOPT_USERPWD, $this->client_id.":".$this->client_secret);
			curl_setopt($ch, CURLOPT_HTTPHEADER, [
				"Content-Type: application/json"
			]);
			
			curl_setopt($ch, CURLOPT_POST, true);
			
			if ($arBodyData !== null) {
			 
				curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($arBodyData));
				
			}
			
			$response = curl_exec($ch);
			
			if (curl_errno($ch)) throw new \Exception('cURL Error: '.curl_error($ch));
			
			$response_data = json_decode($response, true);
			
			return $response_data;
			
		}
		
		public function integrateSDK() {
			
			if ($this->bSandbox) echo '<script src="https://sandbox.paypal.com/sdk/js?client-id='.$this->client_id.'&components=buttons,payment-fields,marks,funding-eligibility&enable-funding=giropay&currency=EUR"></script>';
			else echo '<script src="https://www.paypal.com/sdk/js?client-id='.$this->client_id.'&components=buttons,payment-fields,marks,funding-eligibility&enable-funding=giropay&currency=EUR"></script>';
			
		}
		
		private function getToken(): string {
		
			$ch = curl_init();
			
			curl_setopt($ch, CURLOPT_URL, "https://api-m.sandbox.paypal.com/v1/oauth2/token");
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
			curl_setopt($ch, CURLOPT_USERPWD, "$this->client_id:$this->client_secret");
			curl_setopt($ch, CURLOPT_POST, true);
			curl_setopt($ch, CURLOPT_HTTPHEADER, [
				"Content-Type: application/x-www-form-urlencoded"
			]);
			curl_setopt($ch, CURLOPT_POSTFIELDS, "grant_type=client_credentials");
			
			$response = curl_exec($ch);
			
			if (curl_errno($ch)) throw new \Exception('cURL Error: '.curl_error($ch));
			
			$response_data = json_decode($response, true);
			
			if (!isset($response_data['access_token'])) throw new \Exception('PayPalAPI Error');
			
			return $response_data['access_token'];
		
		}
		
		public static function getInstance(): GiroPayPalAPI {
			
			if (self::$instance === null) {
			
				if (intval(\wpsg_ShopController::getShop()->get_option('wpsg_mod_paypalapi_sandbox')) > 0) {
				
					self::$instance = new GiroPayPalAPI(
						\wpsg_ShopController::getShop()->get_option('wpsg_mod_paypalapi_sandbox_clientid'),
						\wpsg_ShopController::getShop()->get_option('wpsg_mod_paypalapi_sandbox_secret'),
						true,
					);
					
				} else {
					
					self::$instance = new GiroPayPalAPI(
						\wpsg_ShopController::getShop()->get_option('wpsg_mod_paypalapi_clientid'),
						\wpsg_ShopController::getShop()->get_option('wpsg_mod_paypalapi_secret'),
						false,
					);
					
				}
			
			}
			
			return self::$instance;
			
		}
		
	}