<?php

    /**
     * Model für Zugriff auf die Produktgruppen
     */

    class wpsg_productgroup extends wpsg_model {

        static $_tableName = 'WPSG_TBL_PRODUCTS_GROUP';

	    /**
		 * Lädt die Daten der Produktgruppe
		 */
		public function load($productgroup_id) {

			parent::load($productgroup_id);

			$this->data = $this->db->fetchRow("
				SELECT
					PG.*,
					(SELECT COUNT(*) FROM `".WPSG_TBL_PRODUCTS."` AS P WHERE P.`pgruppe` = PG.`id` AND `deleted` != '1') AS `product_count`
				FROM
					`".WPSG_TBL_PRODUCTS_GROUP."` AS PG
				WHERE
					PG.`id` = '".wpsg_q($productgroup_id)."'
			");

			if ($this->data['id'] != $productgroup_id || !wpsg_isSizedInt($productgroup_id)) return false;

			return true;

		}

        public function setImage(int $image_id) {

            $this->setMeta('image', $image_id);

        }

        public function getImage(): int {

            return intval($this->getMeta('image'));

        }

	    public function countProducts()
	    {

		    return $this->product_count;

	    }
	
		/**
		 * @return string
		 */
	    public function getLabel() {
			
			return $this->__get('name');
			
		}

        public function getTemplate() {

			if ($this->data['template_file'] !== '0' && $this->data['template_file'] !== '') $template = $this->data['template_file']; else $template = false;

            return $template;

        }

        /**
         * @param $arFilter
         * @return \wpsg_product[]
         * @throws Exception
         */
        public function findProducts($arFilter = []) {

            $arFilter['productgroup_ids'] = [$this->getId()];

            return \wpsg_product::find($arFilter);

        }
		
		public function hasActiveDiscount(): bool {
			
			if (wpsg_ShopController::getShop()->get_option("wpsg_mod_discount_productgroupdiscount") != "1") {
				
				return false;
				
			} else {
				
				$rabatt = explode("_", $this->__get('rabatt'));
				
				if (sizeof($rabatt) <= 0 || $rabatt[0] != '1' || !preg_match("/\d{2}\.\d{2}\.\d{4}/", $rabatt[1]) || !preg_match("/\d{2}\.\d{2}\.\d{4}/", $rabatt[2]) || !is_float((float)$rabatt[3])) {
	
					return false;
	
				}
		
				$rabatt_value = $rabatt[3];
				$rabatt_start = strtotime($rabatt[1]);
				$rabatt_end = strtotime($rabatt[2]);

				// im Zeitrahmen?
				if (time() >= $rabatt_start && time() <= $rabatt_end) { return true; } else { return false; }
				
			}
			
		}
	    
        /* Statische Funktionen */

	    public static function getProductgroupSelect($arProductFilter = [])
	    {

			if (wpsg_isSizedArray($arProductFilter))
			{

				unset($arProductFilter['limit']);
				unset($arProductFilter['page']);
				unset($arProductFilter['status']);
				unset($arProductFilter['productgroup_ids']);
				unset($arProductFilter['productcategory_ids']);

				list($strQuerySELECT, $strQueryWHERE, $strQueryJOIN, $strQueryHAVING, $strQueryORDER) = wpsg_product::getQueryParts($arProductFilter);

				$strQuery = "
					SELECT
						PG.`id`, CONCAT(PG.`name`,' (', (
							SELECT COUNT(*) FROM `".WPSG_TBL_PRODUCTS."` AS P ".$strQueryJOIN." WHERE (P.`pgruppe` = PG.`id` AND P.`deleted` = 0 ".$strQueryWHERE.")
						), ')') AS `name` ,
						(
							SELECT COUNT(*) FROM `".WPSG_TBL_PRODUCTS."` AS P ".$strQueryJOIN." WHERE (P.`pgruppe` = PG.`id` AND P.`deleted` = 0 ".$strQueryWHERE.")
					  	) AS `product_count`
					FROM
						`".WPSG_TBL_PRODUCTS_GROUP."` AS PG
					HAVING
						`product_count` > 0
					ORDER BY
						`name` ASC
				";

				return $GLOBALS['wpsg_db']->fetchAssocField($strQuery, "id", "name");

			}
			else
			{

				return $GLOBALS['wpsg_db']->fetchAssocField("
					SELECT
						PG.`id`, CONCAT(PG.`name`,' (', (
							SELECT COUNT(*) FROM `".WPSG_TBL_PRODUCTS."` AS P WHERE (P.`pgruppe` = PG.`id` AND P.`deleted` = 0)
						), ')') AS `name` ,
						(SELECT COUNT(*) FROM `".WPSG_TBL_PRODUCTS."` AS P WHERE (P.`pgruppe` = PG.`id` AND P.`deleted` = 0)) AS `product_count`
					FROM
						`".WPSG_TBL_PRODUCTS_GROUP."` AS PG
					HAVING
						`product_count` > 0
					ORDER BY
						`name` ASC
				", "id", "name");

			}

	    } // public static function getProductgroupSelect()

        /**
		 * Zählt die Bestellungen anhand des Filters
		 */
		public static function count($arFilter) {

			list($strQuerySELECT, $strQueryWHERE, $strQueryJOIN, $strQueryHAVING, $strQueryORDER) = self::getQueryParts($arFilter);

			$strQuery = "
				SELECT
					COUNT(*)
				FROM
					(
						SELECT
						  	DISTINCT PG.`id`
						FROM
							`".WPSG_TBL_PRODUCTS_GROUP."` AS PG
							".$strQueryJOIN."
						WHERE
							1
							".$strQueryWHERE."
						HAVING
							1
							".$strQueryHAVING."
					) AS innerSelect
			";

			return $GLOBALS['wpsg_db']->fetchOne($strQuery);

		} // public static function count($arFilter)

		public static function find($arFilter = array())
		{

			list($strQuerySELECT, $strQueryWHERE, $strQueryJOIN, $strQueryHAVING, $strQueryORDER) = self::getQueryParts($arFilter);

			$strLimit = "";

			if (wpsg_isSizedArray($arFilter['limit'])) $strLimit = "LIMIT ".wpsg_q($arFilter['limit'][0]).", ".wpsg_q($arFilter['limit'][1]);

			$strQuery = "
				SELECT
					PG .`id`
					".$strQuerySELECT."
				FROM
					`".WPSG_TBL_PRODUCTS_GROUP."` AS PG
				WHERE
					1
					".$strQueryWHERE."
				HAVING
					1
					".$strQueryHAVING."
				ORDER BY
					".$strQueryORDER."
				".$strLimit."
			";

			$arCustomerID = $GLOBALS['wpsg_db']->fetchAssocField($strQuery);
			$arReturn = array();

			foreach ($arCustomerID as $customer_id)
			{

				$arReturn[$customer_id] = self::getInstance($customer_id);

			}

			return $arReturn;

		} // public function find($arQuery = array())

		public static function getQueryParts($arFilter = array()) {

			$strQuerySELECT = "";
			$strQueryWHERE = "";
			$strQueryJOIN = "";
			$strQueryHAVING = "";

			if (wpsg_isSizedString($arFilter['s'])) $strQueryWHERE .= " AND PG.`name` LIKE '%".wpsg_q($arFilter['s'])."%' ";

			if (wpsg_isSizedString($arFilter['order'], 'name')) { $strQueryORDER = " PG.`name` "; }
			else if (wpsg_isSizedString($arFilter['order'], 'template_file')) { $strQueryORDER = " PG.`template_file` "; }
			else if (wpsg_isSizedString($arFilter['order'], 'product_count')) {

				$strQuerySELECT .= ", (SELECT COUNT(*) FROM `".WPSG_TBL_PRODUCTS."` AS P WHERE P.`pgruppe` = PG.`id`) AS `product_count`) ";
				$strQueryORDER = " `product_count` ";

			}
			else $strQueryORDER = " PG.`pos` ";

			// Richtung
			if (wpsg_isSizedString($arFilter['ascdesc'], "DESC")) $strQueryORDER .= " DESC ";
			else $strQueryORDER .= " ASC ";

			return array($strQuerySELECT, $strQueryWHERE, $strQueryJOIN, $strQueryHAVING, $strQueryORDER);

		} // public function getQueryParts($arFilter = array())

    }